\ elite-event.fs -- Mix and Reference event tracking -- 13Jan13 rjn
\ 
]  here ( *)
\ 
0 [if] \ ----------------------[ 1 Hz Reference ]------------------------------

1. ref-in? executes in ISR routine (see elite-timer2.fs) and toggles every
     500 ms between $ff and $00.
2. The above can be tested with hz-test code in the ISR (see elite-timer2.fs).
3. The value returned by ref-in? is used to synchronize event counting.

[then] \ ----------------------------------------------------------------------
\
\ -----[ events ]
\ 
cpuHERE constant count?      1 cpuALLOT  \ true = in EPUT count interval
cpuHERE constant prev-cp0    1 cpuALLOT  \ holds previous state of cp0 input, 0=low

:m cp-high?  ( - n)   CPT0CN #@  $40 # and m;  \  0=comparator low 
:m /event  cp-high? prev-cp0 #! m;

: cp0-low2high?  ( - ?)  \ non-zero = comparator input went low->high 
   prev-cp0 #@ cp-high? dup prev-cp0 #! xor if drop prev-cp0 #@ ; then drop 0 # ;
\ : cp0-test  cp0-low2high? if drop [ 6 .P0 toggle ] ; then drop ;
\  
\ -----[ interval counter ]
\ 
cpuHERE constant icount  3 cpuALLOT  \ interval counter cells

:m icount-msb  [ icount ] m;
:m icount-mid  [ icount 1 + ] m;
:m icount-lsb  [ icount 2 + ] m;

:m |0icount-msb  0 # icount-msb #! m;
:m |0icount  |0icount-msb  0 # icount-mid #!  0 # icount-lsb #! m;
-: 0icount  |0icount ;
-: !max-icount  $ff #  $ff #  $ff #  icount #  3 # !cells ;

-: icount-
	[ t push  clrc
        	icount-lsb t mov  1 #subb  t icount-lsb mov
                icount-mid t mov  0 #subb  t icount-mid mov
                icount-msb t mov  0 #subb  t icount-msb mov
          t pop ]  ;

:m |@icount  ( - n1 n2 n3)  icount-lsb #@  icount-mid #@  icount-msb #@ m;
: @icount  |@icount ;
\ 
\ -----[ interval setting ]
\ 
cpuHERE constant iset  3 cpuALLOT  \ interval setting

:m iset-msb  [ iset ] m;
:m iset-mid  [ iset 1 + ] m;
:m iset-lsb  [ iset 2 + ] m;

-: 0iset-msb  $00 # iset-msb #! ;
-: 0iset-mid  $00 # iset-mid #! ;
-: 0iset-lsb  $00 # iset-lsb #! ;

0 [if] \ ----- old way
-: /icount   iset-msb #@  icount-msb #!  
             iset-mid #@  icount-mid #!  iset-lsb #@  icount-lsb #! ;
[then] \ -----

:m /icount  
   iset-msb # a! @+ icount-msb #! @+ icount-mid #! @+ icount-lsb #! m; 
\ 
\ -----[ calculated elapsed time in interval ]-----
\  

cpuHERE constant elapsed  3 cpuALLOT  \ interval setting

:m elapsed-msb  [ elapsed ] m;
:m elapsed-mid  [ elapsed 1 + ] m;
:m elapsed-lsb  [ elapsed 2 + ] m;

:m |0elapsed  0 # elapsed-msb #!  0 # elapsed-mid #!  0 # elapsed-lsb #! m;

\ :m |@elapsed  elapsed-lsb #@  elapsed-mid #@  elapsed-msb #@ m;
: @elapsed  elapsed-lsb # a! @ [ 1 dec ] @ [ 1 dec ] @ ;

:m |calc-elapsed
	[ t push  clrc
        	iset-lsb t mov  icount-lsb subb  t elapsed-lsb mov
                iset-mid t mov  icount-mid subb  t elapsed-mid mov
        	iset-msb t mov  icount-msb subb  t elapsed-msb mov
          t pop ]
m;
\
\ -----[ interval setting commands ]
-: !iset  ( lsb mid msb -)   iset #  3 # !cells ;
: 1s        1 #        0 ##  !iset ;
: 10s     $0a #        0 ##  !iset ;
: 100s    $64 #        0 ##  !iset ;
: 1ks     $e8 #  $03 # 0 #   !iset ;
: 10ks    $10 #  $27 # 0 #   !iset ;
: 100ks   $a0 #  $86 # 1 #   !iset ;

-: 1sec$      string " 1 sec" 
-: 10sec$     string " 10 sec"   
-: 100sec$    string " 100 sec"   
-: 1000sec$   string " 1000 sec"  
-: 10ksec$    string " 10K sec"  
-: 100ksec$   string " 100K sec"
-: noset$     string " not set" 

: i   iset-lsb #@  
   |1xor   0=if drop 1sec$    ; then |1xor
   |10xor  0=if drop 10sec$   ; then |10xor
   |$64xor 0=if drop 100sec$  ; then |$64xor
   |$e8xor 0=if drop 1000sec$ ; then |$e8xor
   |$10xor 0=if drop 10Ksec$  ; then |$10xor
   |$a0xor 0=if drop 100ksec$ ; then drop noset$ ;

-: /interval  100s ; 
\ 
\ -----[ event counter ]
\ 
cpuHERE constant ecount 3 cpuALLOT  \ holds total event count

:m ecount-msb    ecount       m;
:m ecount-mid  [ ecount 1+  ] m;
:m ecount-lsb  [ ecount 2 + ] m;

:m |0ecount  0 # ecount-msb #!  0 # ecount-mid #!  0 # ecount-lsb #! m;
:m |@ecount   ecount-lsb #@   ecount-mid #@  ecount-msb #@ m;

:m |ecount+   [ t push  clrc
                 ecount-lsb t mov  1 #addc  t ecount-lsb mov
                 ecount-mid t mov  0 #addc  t ecount-mid mov
                 ecount-msb t mov  0 #addc  t ecount-msb mov
                t pop ]  m;
\ 
0 [if] \ -------------------[ EPUT Counting ]----------------------------------
\ 
1. ~hz toggles P0.7 once per second in ISR (via decade count chain)
2. It also decrements the interval counter during a counting interval.
3. The count? flag is set by the n command to start a new count interval.
   The flag is used in ~hz? to decrement the interval counter on each low to
   high transition of the hertz signal (i.e., when hz? flag changes to high).
4. ?count decrements the interval counter on low to high Hertz transitions
   if the count? flag is true. 
5. ?stop stops the counting interval when the interval counter is counted 
   down to zero.  It does this by clearing the count? flag.
6. It may not be obvious that the count will be accurate.  Here is the
   sequence:
	a. The interval count is loaded into the interval counter.
	b. The event count is zeroed.
	c. The count? flag is set when the next high to low transition
           on the hz? signal occurs.
	d. At this point, events may be accumulated in the ISR by ?event
	   but the interval count will not be decrement until the next 
	   low to high change of the hz? signal.  This is ok. 
	   On the first transition, one second will have elapsed and the 
	   events during that period will have accumulated.
	e. When the final low to high transition of hz? is detected by ~hz?
           (icount is zero), the count flag is zeroed, preventing further
           countdown and shutting down the event counting by ?event.  
	f. At this point, the exact number of interval seconds have been
           counted (e.g., for an interval of 100 seconds, exactly 100
           seconds will have expired since counting was enabled).
	g. In the ISR, ~hz? occurs before ?event.  Thus, when the exact
           interval has been counted down and counting is disabled by
	   zeroing the count? flag, no further events are counted by
	   ?event.
	h. It helps to draw a diagram and annotate it showing when various
	   things happen with respect to low to high transitions of the
	   hz? signal.
7. Note that counts may vary for very fast events (near or exceeding
   10 kHz) and a count may be missed for very slow event rates 
   (e.g., those near or exceeding the interval).  Unlike EPUT, only the
   interval transition is synchronized -- the routines do not care about
   event edges, only that events are shut down at the trailing edge of 
   the count interval.
8. The ISR behaves as follows:
	a. Updates all counters -- this is needed to derive the one Hertz
           signal and detect a low to high transition of the one Hertz signal.
	b. Interval counting also managed by ~hz? .
	c. ~hz? also manages event counting by checking for a zeroed 
	   interval counter.  If it is zero, the count? flag is zeroed to
	   disable further event counting.
	d. If interval counting is in progress, ?event checks for a low
           to high transition on the comparator output and, if one has
           occurred, it bumps the event count.
	e. The ISR is set up and enabled in main and runs continuously
	   thereafter.  Thus, a one Hertz signal appears at P0.7 whenever
	   the application is active.
\  
[then] \ ----------------------------------------------------------------------
\ 
cpuHERE constant hz?        1 cpuALLOT  \ toggles between $ff and 0 at one Hz
cpuHERE constant count?     1 cpuALLOT  \ true = EPUT counting

:m 0count?   0 # count? #! m;
: /hz?  7 .P0 if. $ff # hz? #! ; then 0 # hz? #! ;

:m ?stop  @icount ior ior 0=if drop 0count? ; then drop m;
: ?count  count? #@ if drop icount- ?stop ; then drop ;

-: ~hz?   [ 7 .P0 toggle ]  hz? #@  $ff # xor  dup hz? #!
   if ( L2H transition) drop ?count ; then drop ; 
\ 
\ -----[ ISR event & interval counting ]
\ 
\ -: ?event  event-low2high? if drop |ecount+ [ 7 .P0 toggle ]  ; then drop ;
-: ?event   cp0-low2high? count? #@ and if drop |ecount+ ; then drop ;
\ 
\ -----[ Start EPUT Measurement, s command ]
\ 
: n  \ new interval command 
   0count?  \ shut counting down, just in case n is executed during a count  
   |0ecount \ zero the event counter 
   /icount  \ init the countown interval
\ synchronize counting with the ISR -- wait for L2H on hz?, may take a sec.
   0 # begin drop hz? #@ 0=until drop  0 # begin drop hz? #@ until drop
   $ff # count? #!  \ now its ok to start counting
;
\ 
0 [if] \ --------------[ test sync, cp0 transitions ]--------------------------
\ set signal generator input to a low rate such as 0.1 Hertz
\ display should show a dot for each low to high transition seen on the scope
\ works -- 13Jan13 rjn
-: nt$  string " sync complete!"
: (ntest)   cp0-low2high? if drop [ char . ] # emit ; then drop ;
: ntest   cr
    0 # begin drop hz? #@ 0=until drop  0 # begin drop hz? #@ until drop
    nt$ cr  [ char : ] # emit space  begin (ntest) again ;
[then] \ ----------------------------------------------------------------------
\ 
\
0 [if] \ --------------------[ EPUT Display ]----------------------------------
\ 
1. The current/last EPUT result is displayed before each prompt.
2. eprompt displays the current/last result -- it is patched into prompt code
\  
[then] \ ----------------------------------------------------------------------
\ 
:m ecount>tria  0 # tria #!  
   ecount-msb #@ tria1 #!  ecount-mid #@ tria2 #!  ecount-lsb #@ tria3 #! m;
:m elapsed>tria  0 # tria #!  
   elapsed-msb #@ tria1 #!  elapsed-mid #@ tria2 #!  elapsed-lsb #@ tria3 #! m;
: q  ( esync  250 # us ) |calc-elapsed 
  ecount>tria  tria. [ char : ] # emit
  elapsed>tria tria. ;  

here ( *) epatch org q ( *) org  \ patch display routine into emit>

-: /events  0count?  /interval  /event  |0icount  |0ecount  |0elapsed  ;

( *) here [ swap - cr .( elite-event         ) . .( bytes) ]
\ 
\ 
0 [if] \ ---------------------[ Revision History ]-----------------------------

Date	By  Description
======= === ===================================================================
13Jan13 rjn verified that counting words work.
12Jan13 rjn improved counting code, improved documentation
08Jan13 rjn improved algorithms, implemented "n" command
04Jan13 rjn revised -- just synchs first second in interval
29Dec12 rjn copied from EPUT
04Dec09 rjn moved ?update-eput out of the interval counting conditional
09Nov09 rjn initial version for interrupt counting

[then] \ ----------------------------------------------------------------------

