\ elite-timer2.fs -- elite timer 2 interrupt -- 28Dec12 rjn
\
here  \ ***** start marker
\
\ -----[ interrupt enable/disable ] 
:m |disable-interrupts  [ 7 .IE clr ] m;
:m |enable-interrupts   [ 7 .IE set ] m;
\ : enable-interrupts   |enable-interrupts ;
\ : disable-interrupts  |disable-interrupts ;

:m |t2-run   [ 2 .TMR2CN set ] m;
:m |t2-stop  [ 2 .TMR2CN clr ] m;

:m |t2-enable  [ 5 .IE set ] m;
:m |t2-disable [ 5 .IE clr ] m;

:m |t2-clear-overflow   [ 7 .TMR2CN clr ] m;
\ 
\ 
0 [if] --------------------[ ISR Setup & Timing ]------------------------------

1. The ISR directly toggles the one Hertz output.  All other signals are 
   counted down from direct cells in decade intervals.
------------------------------------------------------------------------ [then]
\
cpuHERE constant dec1  1 cpuALLOT  \ first decade
cpuHERE constant dec2  1 cpuALLOT  \ second decade
cpuHERE constant dec3  1 cpuALLOT  \ third decade
cpuHERE constant dec4  1 cpuALLOT  \ fourth decade
\  
\ -----[ decade counting ]
\ 
:m /dec1  10 # dec1 #! m;
:m /dec2  10 # dec2 #! m;     \ 100 Hz
\ :m /dec2  20 # dec2 #! m;    \  50 Hz
:m /dec3  10 # dec3 #! m;
:m /dec4  10 # dec4 #! m;

: /decades  /dec1  /dec2  /dec3  /dec4  ;

:m |?dec4   [ dec4 dec ] dec4 #@ 0=if ~hz?   /dec4 then drop m;  \ 1 Hz

:m |?dec3   [ dec3 dec ] dec3 #@ 0=if |?dec4 /dec3 then drop m;  \ 10 Hz

:m |?dec2   [ dec2 dec ] dec2 #@ 0=if |?dec3 /dec2 then drop m;  \ 100 Hz

:m |?decs   [ dec1 dec ] dec1 #@ 0=if |?dec2 /dec1 then drop m;  \ 1 kHz

TIM2 interrupt : timer2-isr
   |t2-clear-overflow
   [ t push  ]
\   [ 7 .P0 toggle ]   \ 10 kHz reference out   
   |?decs  \ update decade counters
   ?event  \ toggle P0.7 to verify event logic
   [ t pop ]
   reti

: /timer2  
   /decades  /hz?
   $fb # TMR2RLH #!  $33 # TMR2RLL #!  \ 9.9980 kHz 
   $20 # CKCON  #!  \ set low byte clock source to SYSCLK
     0 # TMR2CN #!  \ 16-bit auto reload, clr overflow flag, T2 stopped
   |t2-enable  |t2-clear-overflow  |t2-run  |enable-interrupts 
;

here  \ ***** stop marker
[ swap -  cr .( elite-timer2.fs     ) . .( bytes) ]

\ 
\ 
0 [if] \ -------------------[ Revision History ]-------------------------------

Date	By  Description
======= === ===================================================================
10Jan13 rjn saved/restored TOS, simplified hz? toggling
28Dec12 rjn cloned from DECUS T2 routine (works, freq off X2).
13Dec10 rjn changed |t2-clear-overflow to use bit-addressable version
05Sep10 rjn final version, verified all outputs -- no jitter on any
04Sep10 rjn working version but with jitter
02Sep10 rjn initial version
[then] \ ----------------------------------------------------------------------

