\ init.fs -- I/O initialization for elite's AGEN module -- 25Dec12 rjn
\
]  \ target forth

here  \ ***** start marker
\
0 [if] ---------------------------[ Notes ]------------------------------------

1. Configures 300 chip on an AGEN Module.  Uses the AGEN Programming and Test
   Platform for programming.
2. To be called as part of cold so that after a reset the hardware 
    returns to these settings
3. If a bit needs changing in the application, "and/orl" it.
4. 300/600 DEV I/O assignments (SOIC14):

   Port	Pin Type Function
   ----	--- ---- --------------------------------------------------------------
   P0.0	  5   PP SYSCLK
   P0.1	  6   PP negative input to comparator
   P0.2   8   PP positive input to comparator ( XTAL1 later?)       
   P0.3	 10   PP output of comparator (XTAL2 later?)
   P0.4	 12   PP RS-232 TX
   P0.5	 13   DI RS-232 RX
   P0.6   1   PP GP output
   P0.7	  2   PP GP output
          3   -- GND
          7   -- VDD (3.3 Volts)
         14   -- RST/C2CK
   4, 9, 11   -- N/C  
[then] \ ----------------------------------------------------------------------
\ 
\ -----[ I/O Assignments ]
\ --- port 0
\ note: macros aren't compiled until used so no need to patch them out
:m sysclk-out  [ 0 .P0 ] m;  \ SYSCLK
:m ~00  [ 0 .P0 toggle ] m;
:m ~01  [ 1 .P0 toggle ] m;
:m ~02  [ 2 .P0 toggle ] m;
:m ~03  [ 3 .P0 toggle ] m;
:m ~04  [ 4 .P0 toggle ] m;
:m ~05  [ 5 .P0 toggle ] m;
:m ~06  [ 6 .P0 toggle ] m;
:m ~07  [ 7 .P0 toggle ] m;
:m ~P0  $ff # P0 xor! m; 

\  
\ 
0 [if] \ -------------------[ I/O initialization ]-----------------------------

 1. Steps for port I/O initialization, per datasheet:
	Step 1 - Select input mode (analog or digital) for all port pins
		 using PnMDIN.  If the pin is in analog mode, a "1" must also
		 be written to the corresponding Port Latch.  Default for
		 PnMDIN is $ff (pins not configured as analog inputs).
	Step 2 - Select the output mode using PnMDOUT.  
		  Output Modes - 0=open drain, 1=push pull
	Step 3 - Assign port pins to desired peripherals using XBRn registers.
	Step 4 - Enable the crossbar (XBARE="1")
2. Pins used as comparator or ADC inputs should be configured as analog inputs.
3. Analog input pins should be skipped by the crossbar.
4. All analog pins must have a "1" set in the corresponding Port Latch reg.
[then] \ ----------------------------------------------------------------------
\ 
\
: /chip 
\ -----[ disable crossbar ] 
   $00 # XBR2 #!  \ disable xbr
\ 
\ -----[ configure I/O ] 
\ 
true [if] \ ----------[ external xtal, 24.5 MHz clock (future) ]----------------
\ --- P0MDIN, P0MDOUT & P0SKIP sequencing per 300 manual
   $f3 ~# P0      and!  \ force crystal pins low, per Step 1  
   $0f ~# P0MDIN  and!  \ P0 pins 2,3 as analog inputs for XTAL, Step 2
			\    pins 0,1 are analog inputs for CP0
   $d0  # P0MDOUT #!    \ Push/Pull P0[4,6,7], P0[4] for RS-232 TX out
   $f3 ~# P0      ior!  \ release crystal pins
   $67  # OSCXCN  #!    \ external oscillator control
\       \ XOSCMD2-0(6-4)=110 - mode is xtal, no divide
\       \ XFCN2-0(2-0)=111 - 10 to 30 MHz xtal
   1 # delay765         \ delay > 1 ms (~8ms @ 24.5 MHz) for xtal to settle
\ Note: The xtal is settled when XTLVLD, P0.7 in OSCXCN is a "1".
\   0 # begin OSCXCN (#@) $80 # and until \ make sure xtal has settled
   $08 # OSCICN #!      \ oscillator control
			   \ CLKSL=1; IOSCEN=0 -- ext. crystal clock
\  $0c # XBR0 #!        \ skip P0.2, 0.3 for xtal osc.
[then] \ ----------------------------------------------------------------------
\  
\ -----[ XBR0 setup ]  
\ 
\   $06  # XBR0 #!       \ skip p0.1 & P0.2 for comparator inputs
			 \    CP0 out on P0.0
    $0f  # XBR0 #!       \ skip p0.0 & P0.1 for comparator inputs
			 \ skip P0.2, P0.3 for xtal osc. (CP0 appears on P0.6)
\ 
\ -----[ comparator setup ]
\   $01 # CPT0MX #!       \ comparator inputs on P0.1 & P0.2
\			 \   negative on P0.1 (gnd), Positive on P0.2
   $00 # CPT0MX #!       \ comp. inputs on P0.0 & P0.1
			 \   negative on P0.1 (gnd), Positive on P0.0
   $85 # CPT0CN #!       \ comp. ctl, enable CP0, 5 mV hysteresis
   $00 # CPT0MD #!       \ comparator response time ($00=fastest)
\ 
\ -----[ XBR1 setup ]
    $13 # XBR1 #!        \ CPO to port pin, UART enabled
\   $18 # XBR1 #!        \ CP0, SYSCLK to port pin
\   $1b # XBR1 #!        \ UART enabled, SYSCLK, CP0 to port pin
\   $03 # XBR1 #!        \ UART enabled, SYSCLK not routed to P0.0
\ 
\ -----[ enable crossbar ]
   $c0 # XBR2 #!        \ enable XBR, weak pullups disabled
\ 
\ -----[ init application I/O ]
\ 
;

here  \ ***** stop marker
[ swap -  cr .( init                 ) . .( bytes) ]

\
\
0 [if] ---------------------[ Revision History ]-------------------------------

Date	By  Description
======= === ===================================================================
25Dec12 rjn Initial version for elite application 

[then] \ ----------------------------------------------------------------------

