\ tri-digits.fs -- convert a triple to decimal digits -- 09Jan13 rjn
\ 
]  here ( *)
\ 
\ -----[ decimal digit extraction from a triple-byte number ]
\ 
0 [if] \ --------------------[ Decimal Conversion ]----------------------------

1. Maximum unsigned value held in 3 bytes = 2**24 = 16,777,215

2.    ----- Decimal/Hex equivalents -----

 	   Decimal      Hexadecimal    
        =============   ==============
           10,000,000     98 96 80
            1,000,000     0F 42 40
              100,000     01 86 A0
               10,000     00 27 10             
                1,000     00 03 E8
                  100     00 00 64
                   10     00 00 0A
                    1     00 00 01

3. Assumes number to be converted is in tria and trib contains value of
   the current digit being converted.  Although tria is only 3 significant
   digits, it is allocated as a quad because of arithmetic operations with
   trib, another triple number.

4. About 350 bytes, 7 direct cells.

5. Charley's digit display words, such as u., are much better from a coding 
   standpoint but are much slower.  I am not sure how important speed is
   for this application, but had this rountine coded and working.

6. BTW -- Considered checking the most significant byte and, if it is
   zero, using ud. .

[then] \ ----------------------------------------------------------------------
\
\ -----[ general purpose arithmetic registers, tria & trib ]
\
cpuHERE constant tria 4 cpuALLOT   \ number to be converted, really a quad
\ [ cr tria . cr ]  \ location 10
:m tria1  [ tria 1 + ] m;
:m tria2  [ tria 2 + ] m;
:m tria3  [ tria 3 + ] m;

\ requires xcells.fs
-: !tria  ( n1 n2 n3 n4 - )   tria #  4 # !cells ;
:m |0tria  0 ## 0 ##  !tria m;
-: 0tria  |0tria ;
\ : .tria   tria #  4 # .cells ;

cpuHERE constant trib-msb 3 cpuALLOT   \ value of current digit being converted
:m trib-mid    [ trib-msb 1 + ] m;
:m trib-lsb    [ trib-msb 2 + ] m;

-: !trib  ( n1 n2 n3 - )  trib-msb #  3 # !cells  ;

:m |0trib  0 ## 0 #  !trib m;
-: 0trib   |0trib ;
\ : .trib   trib-msb #  3 # .cells ;
\ 
\ -----[ Add, Subtract trib to/from tria ]
\
:m |tria+trib   [ t push  clrc
                   tria3 t mov  trib-lsb addc  t tria3 mov
        	   tria2 t mov  trib-mid addc  t tria2 mov
                   tria1 t mov  trib-msb addc  t tria1 mov
        	   tria  t mov         0 #addc t tria  mov
                  t pop ] m;

:m |tria-trib   [ t push  clrc
                   tria3 t mov  trib-lsb subb   t tria3 mov
        	   tria2 t mov  trib-mid subb   t tria2 mov
        	   tria1 t mov  trib-msb subb   t tria1 mov
        	   tria  t mov         0 #subb  t tria  mov
                  t pop ] m;

\
-: tria+trib  |tria+trib ;
-: tria-trib  |tria-trib ;
\ 
\ -----[ number conversion ]
\
-: !digit   $80 # $96 # $98 #  !trib ;
-: !digit1  $40 # $42 # $0F #  !trib ;
-: !digit2  $A0 # $86 # $01 #  !trib ;
-: !digit3  $10 # $27 # $00 #  !trib ;
-: !digit4  $E8 # $03 # $00 #  !trib ;
-: !digit5  $64 # $00 # $00 #  !trib ;
-: !digit6  $0A # $00 # $00 #  !trib ;
-: !digit7  $01 # $00 # $00 #  !trib ;

-: (tdigit)  ( - n)  
  -1 # dup begin drop 1+ tria-trib tria #@ -until drop tria+trib ;
\ 
0 [if] \ --------------------------[ Notes ]-----------------------------------
 
 1. ?0lshow filters out leading zeroes in the output.
 2. Zeros are ignored (e.g., by ?0lshow) until the first non-zero value 
     is encountered.  Then, a flag is set to indicate that all following 
     zeros are to be output.
 3. The calling routine must handle the "all zero" case 
    (e.g., by always showing the last digit).

[then] \ ----------------------------------------------------------------------
\ 
\ filter out leading zeroes in serial output
:m ?0sshow  ( ? n - ?)  |over  if drop (digit) emit ; then drop drop m; 
-: ?0sdigit  ( ? n - ?)    dup if drop (digit) emit drop $ff # ; then drop ?0sshow ;

-: sdigit  ( ? -)  (tdigit) ?0sdigit ; 

\ : tri>serial  \ send tria digits to serial port  
-: tria.  \ send tria digits to serial port
   0 #  !digit  sdigit  !digit1 sdigit  !digit2 sdigit  !digit3 sdigit
        !digit4 sdigit  !digit5 sdigit  !digit6 sdigit  
        !digit7 (tdigit) (digit) emit    \ always display last digit
   drop ;

-: t.  ( lsb mid msb -)   0 # !tria  tria. ; 
\ 
\
0 [if] \ ----------------[ debug, test triple display ]------------------------
\ 
: t9                        |0tria ;  \ 00000000
: t8   $01 # 0 # 0 ##        !tria ;  \ 00000001
: t7   $0C # 0 # 0 ##        !tria ;  \ 00000012
: t6   $7B # 0 # 0 ##        !tria ;  \ 00000123 
: t5   $D2 # $04 # 0 ##      !tria ;  \ 00001234 
: t4   $39 # $30 # 0 ##      !tria ;  \ 00012345 
: t3   $40 # $E2 # $01 # 0 # !tria ;  \ 00123456 
: t2   $87 # $D6 # $12 # 0 # !tria ;  \ 01234567 
: t1   $4E # $61 # $BC # 0 # !tria ;  \ 12345678 
\  
[then] \ ---------------------------------------------------------------------- 

( *) here [ swap -  cr .( tri-digits.fs       ) . .( bytes) ]

\ 
\ 
0 [if] ------------------------[ revisions ]-----------------------------------

Date	By  Comment
=======	=== ===================================================================
11Jan13 rjn verified with t1 - t9 and tria.
09Jan13 rjn adapted EPUT version for ELITE -- eliminated LCD display
24Apr09 rjn slowed down emit using emit+ 
12Nov09 rjn corrected bug in ?0ldigit and added tri>serial.
10Nov09 rjn initial version adapted from pentc.fs

[then] \ ----------------------------------------------------------------------
